#    This is a part of the "Prototype Tagger", an unsupervised POS tagger developed
#    in the Hebrew University by Omri Abend, Roi Reichart and Ari Rappoport.
#    Copyright (C) 2010  Omri Abend, Roi Reichart and Ari Rappoport
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#    If you have any questions, feel free to contact us in:
#    omria01@cs.huji.ac.il, roiri@cs.huji.ac.il
#
#    Please read the README file for more details.
#
#!/usr/local/bin/python
import morpho_preprocessing, morpho_clustering_inducer, commands, sys, parameters_dast, gzip, os, pickle

# Wraps the whole morphology extraction process.
class morfessor_wrapper:
    
    def __init__(self, settings):
        self._settings = settings
        
    def run(self):

        # Preprocessing.
        preprocessor = morpho_preprocessing.morphological_preprocessor(self._settings)
        uppercase_words, non_alphanum = preprocessor.prepare_morpho_files()
        
        morfessor_dir = self._settings.get_string("MORFESSOR_DIRECTORY")+"/train/"
        morfessor_input_filename = self._settings.get_string("MORFESSOR_FILENAME")
        
        gzipped_file = gzip.open(morfessor_input_filename+".gz", "wb")
        gzipped_file.writelines(open(morfessor_input_filename,'rb'))
        gzipped_file.close()
        morfessor_input_filename = morfessor_input_filename + ".gz"
        
        # Running Morfessor.
        self.modify_makefile(morfessor_dir + "Makefile", os.getcwd() + "/" + morfessor_input_filename, 
                             self._settings.get_string("PPL_THRESH"))
        os.rename(morfessor_dir + "Makefile.tmp", morfessor_dir + "/Makefile")
        
        old_dir = os.getcwd()
        os.chdir(morfessor_dir)
        os.system(self._settings.get_string("MAKEFILE_DIR")+"/make")
        os.chdir(old_dir)
        
        morfessor_output_filename = morfessor_dir + "segmentation.final"
        gzipped_file = gzip.open(morfessor_output_filename+".gz", 'rb')
        gunzipped_file = open(morfessor_output_filename,"wb")
        gunzipped_file.write(gzipped_file.read())
        gzipped_file.close()
        
        # Inducing the morphological clustering.
        inducer = morpho_clustering_inducer.morpho_clustering_inducer(self._settings)
        morpho_clustering = inducer.run(morfessor_output_filename, uppercase_words, non_alphanum)
        
        old_dir = os.getcwd()
        os.chdir(morfessor_dir)
        os.system(self._settings.get_string("MAKEFILE_DIR")+"/make realclean")
        os.chdir(old_dir)
        
        # Outputting the morphological clustering file.
        output_f = open(self._settings.get_string("MORPHO_CLUSTERING_FILE"), "w")
        pickle.dump(morpho_clustering, output_f)
        output_f.close()
        
    def modify_makefile(self, makefile_file, morfessor_output_filename, ppl_thresh):
        f = open(makefile_file)
        output_makefile = open(makefile_file+".tmp","w")
        for line in f:
            line = line
            if line.startswith("GZIPPEDINPUTDATA"):
                line = "GZIPPEDINPUTDATA = " + morfessor_output_filename + "\n"
            elif line.startswith("PPLTHRESH"):
                line = "PPLTHRESH = " + ppl_thresh + "\n"
            output_makefile.write(line)
        output_makefile.close()
        f.close()
        
# END OF CLASS
