#    This is a part of the "Prototype Tagger", an unsupervised POS tagger developed
#    in the Hebrew University by Omri Abend, Roi Reichart and Ari Rappoport.
#    Copyright (C) 2010  Omri Abend, Roi Reichart and Ari Rappoport
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#    If you have any questions, feel free to contact us in:
#    omria01@cs.huji.ac.il, roiri@cs.huji.ac.il
#
#    Please read the README file for more details.
#
#!/usr/local/bin/python
import re, sys

class morphological_preprocessor():
    
    # Required settings: the PPLTHRESH of the Morfessor, the text file and whether to use capitalization.
    def __init__(self, settings):
        self._plain_text_file = settings.get_string("INPUT_CORPUS")
        self._output_for_morfessor_filename = settings.get_string("MORFESSOR_FILENAME")
        #self._uppercase_output_file = settings.get_string("UPPERCASE_LIST_FILENAME")
        #self._nonalphanumeric_output_file = settings.get_string("NON_ALPHANUM_FILENAME")
    
    # Should return a list of words that tend to be upper case, a list of words
    # that have non-alphanumeric characters and an input file for morfessor.
    def prepare_morpho_files(self):
        corpus_f = open(self._plain_text_file)
        space_delim = re.compile("\\s+")
        non_alphanumeric_re = re.compile("[^0-9,A-Za-z]")
        
        word_count = 0
        upper_count = dict()
        total_count = dict()
        regular_capitalization_count = dict() 
        
        for line in corpus_f:
            words = space_delim.split(line.strip())
            words = words[1:]
            for word in words:
                word_count += 1
                if word[0].isupper():
                    upper_count[word.lower()] = upper_count.get(word.lower(), 0) + 1
                total_count[word.lower()] = total_count.get(word.lower(), 0) + 1
                regular_capitalization_count[word] = regular_capitalization_count.get(word, 0) + 1
        corpus_f.close()
        
        upper_words = []
        for word in upper_count.keys():
            if 1.0 * upper_count[word] / total_count[word] >= 0.5:
                  upper_words.append(word)
        
        non_alphanum = []
        output_for_morfessor_filename = open(self._output_for_morfessor_filename, "w")
        for word in regular_capitalization_count.keys():
            if non_alphanumeric_re.search(word) != None:
                non_alphanum.append(word)
            else:
                output_for_morfessor_filename.write(str(regular_capitalization_count[word]) + " " + word+"\n")
            
        output_for_morfessor_filename.close()
        
        return upper_words, non_alphanum
        #pickle_variable(upper_words, self._uppercase_output_file)
        #pickle_variable(non_alphanum, self._nonalphanumeric_output_file)
# END OF CLASS
