#    This is a part of the "Prototype Tagger", an unsupervised POS tagger developed
#    in the Hebrew University by Omri Abend, Roi Reichart and Ari Rappoport.
#    Copyright (C) 2010  Omri Abend, Roi Reichart and Ari Rappoport
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#    If you have any questions, feel free to contact us in:
#    omria01@cs.huji.ac.il, roiri@cs.huji.ac.il
#
#    Please read the README file for more details.
#
#!/usr/local/bin/python
# A data structure which holds all the parameter values
import re

class parameters_dast():
    
    def __init__(self, file_name):
        self._param_dict = dict()
        
        delimiter = re.compile("\\s+")
        f = open(file_name)
        for line in f:
            if line.strip().startswith("#") or line.strip() == "":
                continue
            name_val = delimiter.split(line.strip())
            self._param_dict[name_val[0]] = name_val[1] 
        f.close()
        
    def get_string(self, param_name):
        return self._param_dict[param_name]    
        
    def get_int(self, param_name):
        return int(self._param_dict[param_name])
        
    def get_float(self, param_name): 
        return float(self._param_dict[param_name])
    
    def get_list(self, param_name):
        val = self._param_dict[param_name]
        delim = re.compile(",")
        fields = delim.split(val.strip())
        return fields
    
    def contain(self, param_name):
        return param_name in self._param_dict
    
    def set_string(self, param_name, val):
        self._param_dict[param_name] = val
    
# END OF CLASS



