#    This is a part of the "Prototype Tagger", an unsupervised POS tagger developed
#    in the Hebrew University by Omri Abend, Roi Reichart and Ari Rappoport.
#    Copyright (C) 2010  Omri Abend, Roi Reichart and Ari Rappoport
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#    If you have any questions, feel free to contact us in:
#    omria01@cs.huji.ac.il, roiri@cs.huji.ac.il
#
#    Please read the README file for more details.
#
#!/usr/local/bin/python
import sys, re, pickle
from utils import *
from parameters_dast import *
from sequencer import *

# Runs the second stage (the mapping stage) of the tagger.
class second_stage_driver:

    def __init__(self, settings):
        self._settings = settings
    
    def run(self, prototypes, first_stage_cluster_lists):
        
        # Reading settings.
        morpho_clustering_filename = self._settings.get_string("MORPHO_CLUSTERING_FILE")
        
        # Reading the dictionary.
        print 'Reading Lexicon'
        dict_f = open(self._settings.get_string("LEXICON_FILE"))
        vector_dict, freq_dict, last_line = read_dictionary(dict_f)
        dict_f.close()
        print 'Done reading lexicon.', len(vector_dict), "entries."
        
        # Updating the dictionary.
        print 'Updating Lexicon'
        update_dictionary_by_clusters(vector_dict, first_stage_cluster_lists)
        print 'Done updating Lexicon'
        
        space_delim = re.compile("\\s+")
        pos_seqeuencer = sequencer()
        
        morpho_mapped_words = []
        all_prototypes = []
        for proto_class in prototypes:
            all_prototypes.extend(proto_class)
        print 'All the prototypes:', all_prototypes #debug
        
        print 'Performing mapping stage.'
        type_mapping = dict()
        word_count = 0
        morpho_clustering = self.read_morpho_clustering(morpho_clustering_filename, vector_dict.keys(),
                                                        vector_dict, all_prototypes)
        for category in morpho_clustering.values():
            word_count += len(category)
            closest_cluster = get_closest_prototype_cluster(category, prototypes, vector_dict)
            for word in category:
                type_mapping[word] = str(closest_cluster)
            morpho_mapped_words.extend(category)
        print 'Done performing mapping stage - '+str(word_count)+' words mapped.'
        
        return type_mapping, freq_dict
    # END OF METHOD
    
    # Reads the morphological clustering and returns the data structure containing
    # that clustering. 
    # word_list - contains the list of words that appear in the dictionary. Every word
    #             which is not included in the morpho clustering will be set to a singleton
    # vector_dict - maps the words to vectors
    # all_prototypes - the list of prototypes which were already mapped.
    def read_morpho_clustering(self, pickle_file_name, word_list, vector_dict, all_prototypes):
        pickle_file = open(pickle_file_name)
        morpho_clustering = pickle.load(pickle_file)
        pickle_file.close()
        
        induced_clustering = dict()
        clustered_words = []
        for cluster_name in morpho_clustering.keys():
            cluster = morpho_clustering[cluster_name]
            initial_reduced_cluster = []
            for word in cluster:
                if word in word_list and not word in all_prototypes:
                    initial_reduced_cluster.append(word)
            
            # The clusters are split in case they are too large.
            split_clusters = split_list(initial_reduced_cluster,
                                        self._settings.get_int("MAX_MORPHO_CLUSTER_SIZE"))
            
            for index in range(len(split_clusters)):
                cur_split_cluster = split_clusters[index]
                cur_cluster_vector_dict = dict([(x,vector_dict[x]) for x in cur_split_cluster])
                
                if len(cur_cluster_vector_dict.keys()) < 2:
                    continue
                else:
                    sub_clusters, number_of_iterations = \
                          average_link_clustering(cur_cluster_vector_dict, self._settings.get_float("BETA"))
                
                for sub_cluster in sub_clusters.items():
                    induced_clustering[cluster_name+"."+str(index)+"."+sub_cluster[0]] = sub_cluster[1] 
                    clustered_words.extend(sub_cluster[1])
            
        for word in word_list:
            if not word in clustered_words:
                induced_clustering["SINGLE "+word] = [word]
        
        return induced_clustering
    # End of method

def average_cos(cluster, vector_dict):
    cur_avg = 0
    for word in cluster:
        for word2 in cluster:
            if word == word2:
                continue
            cur_avg += cos(vector_dict[word], vector_dict[word2])
    cur_avg = cur_avg / (len(cluster)*(len(cluster)-1))
    return cur_avg
# End of method
